//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         DiaryParagraph.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class DiaryParagraph.
//
//////////////////////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "common.h"
#include "diaryparagraph.h"
#include "toolkit.h"
#include "tools.h"

SDiaryParagraph::SDiaryParagraph(void)
{
   TK_TRACER("SDiaryParagraph::SDiaryParagraph(void)");
   TK_LOG0("SDiaryParagraph constructed through default constructor (empty)");
}

SDiaryParagraph::SDiaryParagraph(bool pExplicitTimeStamp,
                                 int  pHours,
                                 int  pMinutes,
                                 char *pText)
{
   TK_TRACER("SDiaryParagraph::SDiaryParagraph(bool pExplicitTimestamp, int pHours, int pMinutes, char *pText)");
   this->mExplicitTimeStamp = pExplicitTimeStamp;
   this->mHours             = pHours;
   this->mMinutes           = pMinutes;
   this->mSeqNo             = 0;
   this->mText              = pText;
}

SDiaryParagraph::~SDiaryParagraph(void)
{
   TK_TRACER("SDiaryParagraph::~SDiaryParagraph(void)");
}

void SDiaryParagraph::SetTimeStamp(int pHours, int pMinutes)
{
   TK_TRACER("SDiaryParagraph::SetTimeStamp(void)");
   this->mHours = pHours;
   this->mMinutes = pMinutes;
} 


SDiaryParagraph *SDiaryParagraph::CreateFromBuffer(char *&lRunner)
{
   TK_TRACER("SDiaryParagraph::CreateFromBuffer(void)");
   int lHours = 0;
   int lMinutes = 0;

   TK_LOG0("Discarding newlines and spaces before paragraph");
   STools::EatWhite(lRunner);

   TK_LOG0("determine stringlength to determine max size of paragraph");
   int lLength = strlen(lRunner);

   TK_LOG0("length smaller than 5? then timestamp not possible");
   bool lExplicitTimeStamp = true;
   if (lLength >= 5)
   {
      TK_LOG0("Next two chars numeric? may be a timestamp, store hours");
      if (lExplicitTimeStamp == true &&
          STools::isDigit(lRunner[0]) == true &&
          STools::isDigit(lRunner[1]) == true)
      {
         TK_LOG0("two digits found, store hours");
         char lHoursString[3];
         strncpy_s(lHoursString, 3, lRunner, 2);
         lHoursString[2] = '\0';
         lHours = atoi(lHoursString);
      }
      else
      {
         lExplicitTimeStamp = false;
      }

      TK_LOG0("Third character a colon? timestamp confirmation");
      if (lExplicitTimeStamp == true &&
          lRunner[2] != ':')
      {
         lExplicitTimeStamp = false;
      }

      TK_LOG0("4th and 5th chars numeric? definitely a timestamp, store minutes");
      if (lExplicitTimeStamp == true &&
          STools::isDigit(lRunner[3]) == true &&
          STools::isDigit(lRunner[4]) == true)
      {
         TK_LOG0("two digits found, store minutes");
         char lMinutesString[3];
         strncpy_s(lMinutesString, 3, lRunner + 3, 2);
         lMinutesString[2] = '\0';
         lMinutes = atoi(lMinutesString);
      }
      else
      {
         lExplicitTimeStamp = false;
      }

      TK_LOG0("if timestamp, discard spaces from 6 up, to find start of text");
      if (lExplicitTimeStamp == true)
      {
         lRunner += 5;
         STools::EatWhite(lRunner);    
      }
   }

   if (strlen(lRunner) == 0)
   {
      return NULL;
   }

   TK_LOG0("copy text until next appearance of two consecutive newlines");
   char lBuffer[MAX_BUFFER];
   char *lIndex = lBuffer;
   while (lRunner[1] != '\0' &&
          lRunner[0] != '\n' &&
          lRunner[1] != '\n')
   {
      *lIndex = *lRunner;
      lIndex++;
      lRunner++;
   }
   if (lRunner[1] == '\0' || lRunner[1] == '\n')
   {
      *lIndex = *lRunner;
      lIndex++;
      lRunner++;
   }
   lIndex[0] = '\0';

   TK_LOG0("create a new SDiaryParagraph with the found info");
   if (lExplicitTimeStamp == false)
   {
      lHours = 0;
      lMinutes = 0;
   }

   TK_LOG0("only make a new paragraph if there is text");
   if (strlen(lBuffer) > 0)
   {
      return new SDiaryParagraph(lExplicitTimeStamp,
                                 lHours, 
                                 lMinutes,
                                 lBuffer);
   }

   return NULL;
}

void SDiaryParagraph::DisplayDump(void)
{
   TK_TRACER("SDiaryParagraph::DisplayDump(void)");
   char lMsg[MAX_BUFFER];
   sprintf_s(lMsg, MAX_BUFFER - 1, "mExplicitTimeStamp: %d\nmHours: %d\nmMinutes: %d\nmSeqNo: %d\nmText:\n%s", 
             this->mExplicitTimeStamp, 
             this->mHours, 
             this->mMinutes,
             this->mSeqNo, 
             this->mText);
   AfxMessageBox(lMsg);
}

bool SDiaryParagraph::comesBefore(SDiaryParagraph *pParagraph)
{
   TK_TRACER("SDiaryParagraph::comesBefore(SDiaryParagraph *pParagraph)");
   if (pParagraph == NULL)
   {
      return false;
   }
   if (pParagraph->getHours() > this->mHours)
   {
      return true;
   }
   else if (pParagraph->getHours() < this->mHours)
   {
      return false;
   }
   else if (pParagraph->getMinutes() > this->mMinutes)
   {
      return true;
   }
   else if (pParagraph->getMinutes() < this->mMinutes)
   {
      return false;
   }
   TK_LOG0("Timestamps are identical, how about sequence number?");
   if (pParagraph->getSequenceNumber() > this->mSeqNo)
   {
      return true;
   }
   else if (pParagraph->getSequenceNumber() < this->mSeqNo)
   {
      return false;
   }
   TK_LOG0("Even sequence numbers match, so order is irrelevant, return false");
   return false;
}

void SDiaryParagraph::Write(char *pOutputBuffer)
{
   TK_TRACER("SDiaryParagraph::Write(char *pOutputBuffer)");
   char lBuffer[MAX_BUFFER];
   lBuffer[0] = '\0';
   if (this->mExplicitTimeStamp == true)
   {
      sprintf_s(lBuffer, MAX_BUFFER - 1, "%02d:%02d  %s\n\n", 
                this->getHours(),
                this->getMinutes(),
                this->mText);
   }
   else
   {
      strcat_s(lBuffer, MAX_BUFFER - 1, this->mText);
      strcat_s(lBuffer, MAX_BUFFER - 1, "\n\n");
   }
   strcat_s(pOutputBuffer, MAX_BUFFER - 1, lBuffer);
}

bool SDiaryParagraph::operator == (SDiaryParagraph &pOther)
{
   TK_TRACER("SDiaryParagraph::SDiaryParagraph(void)");
   if (this->mExplicitTimeStamp == pOther.mExplicitTimeStamp &&
       this->mHours == pOther.mHours &&
       this->mMinutes == pOther.mMinutes &&
       strcmp(this->mText, pOther.mText) == 0)
   {
      TK_LOG0("SDiaryParagraphs equal...");
      return true;
   }
   TK_LOG0("SDiaryParagraphs differ...");
   return false;
}

void SDiaryParagraph::Export(FILE *pFile, CString pDirectory, OutputType pType)
{
    TK_TRACER("SDiaryParagraph::Export(FILE *pFile, CString pDirectory, OutputType pType)");

    // pre-paragraph processing
    switch (pType)
    {
    case enSimpleTextNoImages:
		if (this->mExplicitTimeStamp == true)
		{
			fprintf(pFile, "%02d:%02d  ", 
					this->getHours(),
					this->getMinutes());
		}
	    break;

    case enSimpleHTMLNoImages:
    case enSimpleHTMLInlineImages:
    case enSimpleHTMLLinkedImages:
	    fprintf(pFile, "<p>");
		if (this->mExplicitTimeStamp == true)
		{
			fprintf(pFile, "<b>%02d:%02d</b>  ", 
					this->getHours(),
					this->getMinutes());
		}
		break;

    case enFramedHTML:
 	    break;
    }

	// outputting the paragraph itself
	fprintf(pFile, "%s", this->mText);

	// post-paragraph processing 
    switch (pType)
    {
    case enSimpleTextNoImages:
		fprintf(pFile, "\n\n");
	    break;

    case enSimpleHTMLNoImages:
    case enSimpleHTMLInlineImages:
    case enSimpleHTMLLinkedImages:
	    fprintf(pFile, "</p>");
	    break;

    case enFramedHTML:
 	    break;
    }
}
