//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         DiaryParagraph.h
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Declaration of Class DiaryParagraph.
//
//////////////////////////////////////////////////////////////////////////////////////////////
#ifndef DIARYPARAGRAPH_H_INCLUDED
#define DIARYPARAGRAPH_H_INCLUDED

#include "afxtempl.h"
#include "exportdlg.h"

//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Class SDiaryParagraph
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  SDiaryParagraph is basically any uninterrupted stream of sentences, which 
//                is optionally preceeded by a timestamp. These are used to sort paragraphs
//                from two diary pages based on the times they have. 
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//
//////////////////////////////////////////////////////////////////////////////////////////////
class SDiaryParagraph
{
private:
   bool mExplicitTimeStamp;
   int mHours;
   int mMinutes;
   int mSeqNo;
   CString mText;

public:
   SDiaryParagraph(void);
   SDiaryParagraph(bool pExplicitTimeStamp,
                   int  pHours,
                   int  pMinutes,
                   char *pText);
   virtual ~SDiaryParagraph(void);

   bool hasExplicitTimeStamp(void)     { return this->mExplicitTimeStamp;  };
   void SetSeqNo(int pSeqNo)           { this->mSeqNo = pSeqNo;            };
   int  getHours(void)                 { return this->mHours;              };
   int  getMinutes(void)               { return this->mMinutes;            };
   int  getSequenceNumber(void)        { return this->mSeqNo;              };
   int  getTime(void)                  { return (this->mHours * 100 + this->mMinutes); };

   bool operator == (SDiaryParagraph &pOther);
   void SetTimeStamp(int pHours, int pMinutes);  
   bool comesBefore(SDiaryParagraph *pParagraph);
   void Write(char *pOutputBuffer);
   void Export(FILE *pFile, CString pDirectory, OutputType pType);

   void DisplayDump(void);

   static SDiaryParagraph *CreateFromBuffer(char *&lRunner);

};

#endif
