//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         SettingsFile.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  Implementation of Class SettingsFile.
//
//////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <direct.h>
#include "settingsfile.h"
#include "setting.h"
#include "toolkit.h"

SSettingsFile::SSettingsFile(void)
{
   mFileName = "";
   mSynchronized = false;
}

SSettingsFile::~SSettingsFile(void)
{
   if (!this->mSettings.IsEmpty())
   {
      POSITION lPos = this->mSettings.GetHeadPosition();
   
      while (lPos != NULL)
      {
        SSetting *lSetting = this->mSettings.GetNext(lPos);
        if (lSetting != NULL)
        {
            delete lSetting;
        }
      }
   }
}

void SSettingsFile::SetFileName(char *pFileName)
{
	char thePath[MAX_STRING];
    _getcwd(thePath, MAX_STRING - 1);
	mFileName = thePath;
	mFileName += "\\";
    mFileName += pFileName;
    mSynchronized = false;
    this->ReadFile();
}

void SSettingsFile::ReadFile(void)
{
    FILE *lFile = fopen(this->mFileName.GetBuffer(0), "r");
   if (lFile != NULL)
   {
      TK_LOG1("SSettings File %s opened, reading settings", this->mFileName.GetBuffer(0));
      SSetting *lNewSSetting = NULL;
      while ((lNewSSetting = SSetting::ReadSSetting(lFile)) != NULL)
      {
           this->AddSetting(lNewSSetting);
      }
   }
   else
   {
      TK_LOG1("SSettings File %s cannot be opened", this->mFileName.GetBuffer(0));
   }
}

void SSettingsFile::WriteFile(void)
{
    FILE *lFile = fopen(this->mFileName.GetBuffer(0), "w");
   if (lFile != NULL)
   {
      TK_LOG1("SSettings File %s opened, writing settings", this->mFileName.GetBuffer(0));
      
          if (!this->mSettings.IsEmpty())
       {
          POSITION lPos = this->mSettings.GetHeadPosition();
   
          while (lPos != NULL)
          {
            SSetting *lSetting = this->mSettings.GetNext(lPos);
            if (lSetting != NULL)
            {
                lSetting->WriteSSetting(lFile);
            }
          }
        }
      fclose(lFile);
   }
   else
   {
      TK_LOG1("SSettings File %s cannot be opened", this->mFileName.GetBuffer(0));
   }
}

void SSettingsFile::AddSetting(SSetting *&pNewSetting)
{
   if (pNewSetting != NULL)
   {
      if (!this->mSettings.IsEmpty())
      {
         POSITION lPos = this->mSettings.GetHeadPosition();
   
         while (lPos != NULL)
         {
           SSetting *lSetting = this->mSettings.GetNext(lPos);
           if (lSetting != NULL)
           {
               if (lSetting == pNewSetting || lSetting->GetName() == pNewSetting->GetName())
            {
				if (lSetting != pNewSetting)
				{
                    lSetting->SetValue(pNewSetting->GetValue());
                    delete pNewSetting;
			        pNewSetting = lSetting;
				}
                return;
            }
           }
         }
      }
      this->mSettings.AddTail(pNewSetting);
   }
}

void SSettingsFile::Dump(void)
{
   if (!this->mSettings.IsEmpty())
   {
      POSITION lPos = this->mSettings.GetHeadPosition();
   
      while (lPos != NULL)
      {
        SSetting *lSetting = this->mSettings.GetNext(lPos);
        if (lSetting != NULL)
        {
            lSetting->Dump();
        }
      }
   }
}

SSetting *SSettingsFile::GetSetting(char *pName)
{
   if (!this->mSettings.IsEmpty())
   {
      POSITION lPos = this->mSettings.GetHeadPosition();
   
      while (lPos != NULL)
      {
        SSetting *lSetting = this->mSettings.GetNext(lPos);
        if (lSetting != NULL)
        {
            if (lSetting->GetName() == pName)
          {
             return lSetting;
          }
        }
      }
   }
    return NULL;
}
